#!/usr/bin/python3
###############################################################################
#                                                                             #
# Westferry - The IPFire web user interface                                   #
# Copyright (C) 2015 IPFire development team                                  #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################

import datetime
import collecty.client
import mimetypes
import tornado.web

from . import base
from .. import ui

from ..i18n import N_

class AnalyticsBaseHandler(base.BaseHandler):
	@property
	def menu(self):
		_ = self.locale.translate

		m = ui.menu.Menu(self, _("Analytics"))

		# Overview
		m.add_handler(AnalyticsOverviewHandler)
		m.add_divider()

		# System
		m.add_handler(AnalyticsSystemOverviewHandler)

		return m

	def render_graphs(self, graphs):
		self.render("graphs.html", graphs=graphs)


class AnalyticsOverviewHandler(AnalyticsBaseHandler):
	url = r"/analytics"
	title = N_("Overview")

	def get(self):
		self.render("base.html")


class AnalyticsSystemBaseHandler(AnalyticsBaseHandler):
	@property
	def menu(self):
		_ = self.locale.translate

		m = ui.menu.Menu(self, _("System"))

		# Back...
		m.add_handler(AnalyticsOverviewHandler, title=_("Back..."))
		m.add_divider()

		# Overview
		m.add_handler(AnalyticsSystemOverviewHandler, title=_("Overview"))
		m.add_divider()

		# Most interesting items
		m.add_handler(AnalyticsSystemProcessorsHandler)
		m.add_handler(AnalyticsSystemMemoryHandler)
		m.add_handler(AnalyticsSystemTemperaturesHandler)

		# Others
		s = m.add_submenu(_("More"))
		s.add_handler(AnalyticsSystemEntropyHandler)

		return m


class AnalyticsSystemOverviewHandler(AnalyticsSystemBaseHandler):
	url = r"/analytics/system"
	title = N_("System")

	def get(self):
		self.render("base.html")


class AnalyticsSystemProcessorsHandler(AnalyticsSystemBaseHandler):
	url = r"/analytics/system/processors"
	title = N_("Processors")

	def get(self):
		_ = self.locale.translate

		graphs = [
			ui.graphs.Graph(self, "processor"),
			ui.graphs.Graph(self, "processor-temperature"),
			ui.graphs.Graph(self, "cpufreq"),
			ui.graphs.Graph(self, "loadavg"),
		]

		self.render_graphs(graphs)


class AnalyticsSystemMemoryHandler(AnalyticsSystemBaseHandler):
	url = r"/analytics/system/memory"
	title = N_("Memory")

	def get(self):
		_ = self.locale.translate

		graphs = [
			ui.graphs.Graph(self, "memory"),
		]

		self.render_graphs(graphs)


class AnalyticsSystemTemperaturesHandler(AnalyticsSystemBaseHandler):
	url = r"/analytics/system/temperatures"
	title = N_("Temperatures")

	def get(self):
		_ = self.locale.translate

		graphs = [
			ui.graphs.Graph(self, "sensors-temperature"),
			ui.graphs.Graph(self, "processor-temperature"),
		]

		self.render_graphs(graphs)


class AnalyticsSystemEntropyHandler(AnalyticsSystemBaseHandler):
	url = r"/analytics/system/entropy"
	title = N_("Entropy")

	def get(self):
		_ = self.locale.translate

		graphs = [
			ui.graphs.Graph(self, "entropy"),
		]

		self.render_graphs(graphs)


class GraphExportHandler(base.BaseHandler):
	VALID_INTERVALS = ("hour", "day", "month", "week", "year")
	DEFAULT_INTERVAL = "day"

	SUPPORTED_FORMATS = ("pdf", "png", "svg")

	url = r"/graph/([\w\-]+)(?:/([\w\d\.]+))?\.(%s)" % "|".join(SUPPORTED_FORMATS)

	def get(self, template_name, object_id, format):
		# Get the requested dimensions of the image
		height = self.get_argument_int("height", None)
		width  = self.get_argument_int("width", None)

		# Get the requested interval
		interval = self.get_argument("interval", self.DEFAULT_INTERVAL)
		if interval and not interval in self.VALID_INTERVALS:
			raise tornado.web.HTTPError(400, _("Invalid interval: %s") % interval)

		# Create the graph object
		g = ui.graphs.Graph(self, template_name, object_id=object_id)

		# Generate the graph image
		image = g.generate_graph(format=format.upper(), interval=interval,
			height=height, width=width)

		# Set the HTTP headers
		self._make_headers(format, template_name, object_id)

		# Deliver the content
		self.finish(image.get("image"))

	def _make_headers(self, extension, template_name, object_id):
		# Determine the mime type
		mimetype = mimetypes.types_map.get(".%s" % extension, "application/octet-stream")
		self.set_header("Content-Type", mimetype)

		# Add the timestamp when this graph was generated
		now = datetime.datetime.now()

		# Put together the filename (for downloads)
		filename = [self.backend.system.hostname, template_name, object_id, now.isoformat()]
		filename = "%s.%s" % ("-".join((e for e in filename if e)), extension)

		if extension == "pdf":
			self.set_header("Content-Disposition", "attachment; filename=%s" % filename)
		else:
			self.set_header("Content-Disposition", "inline; filename=%s" % filename)


class GraphHandler(base.BaseHandler):
	url = r"/graph/([\w\-]+)(?:/([\w\d\.]+))?"

	def get(self, template, object_id):
		graph = ui.graphs.Graph(self, template, object_id=object_id)

		self.render("graph.html", graph=graph)
